@@ -1,21 +1,53 @@
 from __future__ import unicode_literals
 
 import json
+import logging
 from urllib import urlencode
 from urlparse import parse_qs
 
 from django.core.urlresolvers import reverse
+from django.utils.crypto import constant_time_compare
 from django.utils.encoding import force_unicode
 
 from requests.api import request
 from requests.auth import OAuth1
 from requests.exceptions import RequestException
 
+try:
+    from django.utils.crypto import get_random_string
+except ImportError: # pragma: no cover
+    # Backport implementation from Django 1.4
+    import hashlib
+    import random
+    import string
+    import time
+    from django.conf import settings
+    try:
+        random = random.SystemRandom()
+        using_sysrandom = True
+    except NotImplementedError:
+        import warnings
+        warnings.warn('A secure pseudo-random number generator is not available '
+                      'on your system. Falling back to Mersenne Twister.')
+        using_sysrandom = False
+
+    def get_random_string(length=12, allowed_chars=string.ascii_letters + string.digits):
+        "Returns a securely generated random string."
+        if not using_sysrandom:
+            # Re-seed random
+            bytes = b"{0}{1}{2}".format(random.getstate(), time.time(), settings.SECRET_KEY)
+            random.seed(hashlib.sha256(bytes).digest())
+        return ''.join([random.choice(allowed_chars) for i in range(length)])
+
+
+logger = logging.getLogger('allaccess.clients')
+
 
 class BaseOAuthClient(object):
     
-    def __init__(self, provider):
+    def __init__(self, provider, token=''):
         self.provider = provider
+        self.token = token
 
     def get_access_token(self, request, callback=None):
         "Fetch access token from callback request."
@@ -25,8 +57,9 @@ class BaseOAuthClient(object):
         "Fetch user profile information."
         try:
             response = self.request('get', self.provider.profile_url, token=raw_token)
-        except RequestException:
-            # TODO: Logging
+            response.raise_for_status()
+        except RequestException as e:
+            logger.error('Unable to fetch user profile: {0}'.format(e))
             return None
         else:
             return response.json if response.json is not None else response.text
@@ -51,6 +84,10 @@ class BaseOAuthClient(object):
         "Build remote url request."
         return request(method, url, **kwargs)
 
+    @property
+    def session_key(self):
+        raise NotImplementedError('Defined in a sub-class') # pragma: no cover
+
 
 class OAuthClient(BaseOAuthClient):
 
@@ -65,8 +102,9 @@ class OAuthClient(BaseOAuthClient):
             try:
                 response = self.request('post', self.provider.access_token_url, 
                                         token=raw_token, data=data, oauth_callback=callback)
-            except RequestException:
-                # TODO: Logging
+                response.raise_for_status()
+            except RequestException as e:
+                logger.error('Unable to fetch access token: {0}'.format(e))
                 return None
             else:
                 return response.text
@@ -77,8 +115,9 @@ class OAuthClient(BaseOAuthClient):
         callback = force_unicode(request.build_absolute_uri(callback))
         try:
             response = self.request('post', self.provider.request_token_url, oauth_callback=callback)
-        except RequestException:
-            # TODO: Logging
+            response.raise_for_status()
+        except RequestException as e:
+            logger.error('Unable to fetch request token: {0}'.format(e))
             return None
         else:
             return response.text
@@ -104,7 +143,7 @@ class OAuthClient(BaseOAuthClient):
 
     def request(self, method, url, **kwargs):
         "Build remote url request. Constructs necessary auth."
-        user_token = kwargs.pop('token', '')
+        user_token = kwargs.pop('token', self.token)
         token, secret = self.parse_raw_token(user_token)
         callback = kwargs.pop('oauth_callback', None)
         verifier = kwargs.get('data', {}).get('oauth_verifier')
@@ -126,9 +165,26 @@ class OAuthClient(BaseOAuthClient):
 
 class OAuth2Client(BaseOAuthClient):
 
+    def check_application_state(self, request, callback):
+        "Check optional state parameter."
+        stored = request.session.get(self.session_key, None)
+        returned = request.GET.get('state', None)
+        check = False
+        if stored is not None:
+            if returned is not None:
+                check = constant_time_compare(stored, returned)
+            else:
+                logger.error('No state parameter returned by the provider.')
+        else:
+            logger.error('No state stored in the sesssion.')
+        return check
+
     def get_access_token(self, request, callback=None):
         "Fetch access token from callback request."
         callback = request.build_absolute_uri(callback or request.path)
+        if not self.check_application_state(request, callback):
+            logger.error('Application state check failed.')
+            return None
         if 'code' in request.GET:
             args = {
                 'client_id': self.provider.key,
@@ -138,23 +194,34 @@ class OAuth2Client(BaseOAuthClient):
                 'grant_type': 'authorization_code',
             }
         else:
+            logger.error('No code returned by the provider')
             return None
         try:
             response = self.request('post', self.provider.access_token_url, data=args)
-        except RequestException:
-            # TODO: Logging
+            response.raise_for_status()
+        except RequestException as e:
+            logger.error('Unable to fetch access token: {0}'.format(e))
             return None
         else:
             return response.text
 
+    def get_application_state(self, request, callback):
+        "Generate state optional parameter."
+        return get_random_string(32)
+
     def get_redirect_args(self, request, callback):
         "Get request parameters for redirect url."
         callback = request.build_absolute_uri(callback)
-        return {
+        args = {
             'client_id': self.provider.key,
             'redirect_uri': callback,
             'response_type': 'code',
         }
+        state = self.get_application_state(request, callback)
+        if state is not None:
+            args['state'] = state
+            request.session[self.session_key] = state
+        return args
 
     def parse_raw_token(self, raw_token):
         "Parse token and secret from raw token response."
@@ -170,7 +237,7 @@ class OAuth2Client(BaseOAuthClient):
 
     def request(self, method, url, **kwargs):
         "Build remote url request. Constructs necessary auth."
-        user_token = kwargs.pop('token', '')
+        user_token = kwargs.pop('token', self.token)
         token, _ = self.parse_raw_token(user_token)
         if token is not None:
             params = kwargs.get('params', {})
@@ -178,10 +245,14 @@ class OAuth2Client(BaseOAuthClient):
             kwargs['params'] = params
         return super(OAuth2Client, self).request(method, url, **kwargs)
 
+    @property
+    def session_key(self):
+        return 'allaccess-{0}-request-state'.format(self.provider.name)
 
-def get_client(provider):
+
+def get_client(provider, token=''):
     "Return the API client for the given provider."
     cls = OAuth2Client
     if provider.request_token_url:
         cls = OAuthClient
-    return cls(provider)
+    return cls(provider, token)
