@@ -1,4 +1,5 @@
 from .test_backends import AuthBackendTestCase
 from .test_clients import OAuthClientTestCase, OAuth2ClientTestCase
-from .test_models import ProviderTestCase
+from .test_context_processors import AvailableProvidersTestCase
+from .test_models import ProviderTestCase, AccountAccessTestCase
 from .test_views import OAuthRedirectTestCase, OAuthCallbackTestCase
